@@ -6,12 +6,17 @@ CommandTrnasHandler 物件為 Loader 內部使用之物件
 
 import argparse
 import enum
+from fileinput import filename
 import math
 import os
+from sqlite3 import Timestamp
+from numpy import int16, int8
 import progressbar
 import time
 import serial
 import sys
+import datetime
+from pathlib import Path
 
 from asaloader import alp
 from asaloader import device
@@ -256,10 +261,33 @@ class CommandTrnasHandler():
     def cmd_v2_ext_flash_read(self):
         pass
 
-    def cmd_v2_ext_flash_earse_all(self):
-        self._put_packet(alp.Command.EXT_FLASH_EARSE_ALL, b'')
+    def cmd_ext_flash_prepare(self, filename):
+        payload = bytes(filename, "ascii")
+        self._put_packet(alp.Command.EXT_FLASH_FOPEN, payload)
         rep = self._get_packet()
-        if rep['command'] == alp.Command.EXT_FLASH_EARSE_ALL and rep['data'][0] == 0:
+        if rep['command'] == alp.Command.EXT_FLASH_FOPEN and rep['data'][0] == 0:
+            self.cmd_ext_flash_hex_delete()
+            self.cmd_ext_flash_prepare(filename)
+            return True
+        elif rep['command'] == alp.Command.EXT_FLASH_FOPEN and rep['data'][0] == 2:
+            return True
+        else:
+            return False
+
+    def cmd_ext_flash_hex_delete(self):
+        self._put_packet(alp.Command.EXT_FLASH_HEX_DEL, b'')
+        rep = self._get_packet()
+        if rep['command'] == alp.Command.EXT_FLASH_HEX_DEL and rep['data'][0] == 0:
+            return True
+        else:
+            return False
+
+    def cmd_ext_flash_finish(self):
+        now = datetime.datetime.now()
+        t_stamp = bytearray([now.minute, now.hour, now.day, now.month, (now.year - 2000)])
+        self._put_packet(alp.Command.EXT_FLASH_FCLOSE, t_stamp)
+        rep = self._get_packet()
+        if rep['command'] == alp.Command.EXT_FLASH_FCLOSE and rep['data'][0] == 0:
             return True
         else:
             return False
@@ -457,6 +485,10 @@ class Loader():
     def prog_time(self):
         return self._prog_time
 
+    @property
+    def ext_flash_file(self):
+        return Path(self._ext_flash_file).stem
+
     def _prepare(self):
         """燒錄前準備函式
 
@@ -660,13 +692,14 @@ class Loader():
         elif self._protocol_version == 2:
             if self._ext_flash_page_idx == 0:
                 # TODO: 修改為刪除特定編號檔案
-                self._cth.cmd_v2_ext_flash_earse_all()
+                self._cth.cmd_ext_flash_prepare(self.ext_flash_file[:8])
             self._cth.cmd_v2_ext_flash_write(address, data)
 
         self._ext_flash_page_idx += 1
         self._cur_step += 1
 
         if self._ext_flash_page_idx == len(self._ext_flash_pages):
+            self._cth.cmd_ext_flash_finish()
             self._stage = next(self._stage_iter)
 
     def _do_ext_to_int_prog_step(self):
@@ -676,7 +709,11 @@ class Loader():
             pass
         elif self._protocol_version == 2:
             # TODO: 修改為燒錄特定編號檔案
-            self._cth.cmd_v2_flash_earse_all()
+            if self._device_type == 5:
+                #asa_m4_v1 takes longer chip erase time
+                self._cth.cmd_v3_flash_earse_all()
+            else:
+                self._cth.cmd_v2_flash_earse_all()
             self._cth.cmd_v2_prog_ext_to_int()
 
     def _do_eep_prog_step(self):
